;;; blinkline.el --- blink a line and keep the line highlighted 

;; $Id: blinkline.el,v 1.2 2002/12/19 04:57:49 burton Exp $

;; Copyright (C) 2000-2003 Free Software Foundation, Inc.
;; Copyright (C) 2000-2003 Kevin A. Burton (burton@openprivacy.org)

;; Author: Kevin A. Burton (burton@openprivacy.org)
;; Maintainer: Kevin A. Burton (burton@openprivacy.org)
;; Location: http://relativity.yi.org
;; Keywords: 
;; Version: 1.0.0

;; This file is [not yet] part of GNU Emacs.

;; This program is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free Software
;; Foundation; either version 2 of the License, or any later version.
;;
;; This program is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License along with
;; this program; if not, write to the Free Software Foundation, Inc., 59 Temple
;; Place - Suite 330, Boston, MA 02111-1307, USA.

;;; Commentary:
;;
;; Blinks the current line in the current buffer until the user gives some
;; input.  This can be used to highlight a section of a buffer that needs
;; user attention.

;; Provides a mechanism for blinking the current line within Emacs

;; NOTE: If you enjoy this software, please consider a donation to the EFF
;; (http://www.eff.org)

;;; Code:

(defface blinkline-face '((t (:background "DarkBlue")))
  "Face used for the last available match when a face isn't specified."
  :group 'blinkline)

(defvar blinkline--overlay nil "Overlay used for blinking lines.")

(defun blinkline(&optional face)
  "Blink the current line and deactivate the blink when anything happens within
Emacs."
  (interactive)
  
  (let((overlay (make-overlay (point-at-bol) (1+ (point-at-eol)) (current-buffer))))

    (when (null face)
      (setq face 'blinkline-face))
    
    (overlay-put overlay 'face face)

    (overlay-put overlay 'priority 1)

    (setq blinkline--overlay overlay)
    
    ;;delete this overlay...
    (add-hook 'pre-command-hook 'blinkline-unblink)))

(defun blinkline-unblink()

  (delete-overlay blinkline--overlay)

  (setq blinkline--overlay nil)
  
  (remove-hook 'pre-command-hook 'blinkline-unblink))

(provide 'blinkline)

;;; blinkline.el ends here