;;; bookmark+.el --- smart modern bookmark operation

;; $Id: bookmark+.el,v 1.3 2003/01/10 05:57:59 burton Exp $

;; Copyright (C) 1997-2000 Free Software Foundation, Inc.

;; Author: Kevin A. Burton (burton@openprivacy.org)
;; Maintainer: Kevin A. Burton (burton@openprivacy.org)
;; Location: http://relativity.yi.org
;; Keywords: 
;; Version: 1.0.0

;; This file is [not yet] part of GNU Emacs.

;; This program is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free Software
;; Foundation; either version 2 of the License, or any later version.
;;
;; This program is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License along with
;; this program; if not, write to the Free Software Foundation, Inc., 59 Temple
;; Place - Suite 330, Boston, MA 02111-1307, USA.

;;; Commentary:

(defface bookmark-face '((t (:background "SeaGreen")))
  "Face used to highlight current line.")

(defun bookmark-bmenu-list ()
  "Display a list of existing bookmarks.
The list is displayed in a buffer named `*Bookmark List*'.
The leftmost column displays a D if the bookmark is flagged for
deletion, or > if it is flagged for displaying."
  (interactive)
  (bookmark-maybe-load-default-file)
  (if (interactive-p)
      (pop-to-buffer (get-buffer-create "*Bookmark List*"))
    (set-buffer (get-buffer-create "*Bookmark List*")))

  (shrink-window-if-larger-than-buffer)

  (let ((buffer-read-only nil))
    (delete-region (point-max) (point-min))
    (goto-char (point-min))             ;sure are playing it safe...

    (insert "% Bookmark                      File\n")
    (insert "- --------                      ----\n")

    (bookmark-maybe-sort-alist)
    (mapcar
     (lambda (full-record)
       ;; if a bookmark has an annotation, prepend a "*"
       ;; in the list of bookmarks.
       (let ((annotation (bookmark-get-annotation
                          (bookmark-name-from-full-record full-record))))
         (if (and annotation (not (string-equal annotation "")))
             (insert " *")
           (insert "  "))
         (let ((start (point)))
           (insert (bookmark-name-from-full-record full-record))
           (if window-system
               (put-text-property start 
                                  (save-excursion (re-search-backward
                                                   "[^ \t]")
                                                  (1+ (point)))
                                  'mouse-face 'highlight))
           (insert "\n"))))
     bookmark-alist))
  (goto-char (point-min))
  (forward-line 2)
  (bookmark-bmenu-mode)
  (if bookmark-bmenu-toggle-filenames
      (bookmark-bmenu-toggle-filenames t)))



(defun bookmark-overlay-create(&optional point)
  "Create an overlay for the full line at point."

  (interactive)
  (if (null point)
      (setq point (point)))

  (let(overlay)
    (setq overlay (make-overlay (save-excursion
                                  (goto-char (point-at-bol))
                                  (point))
                                (save-excursion
                                  (goto-char (point-at-eol))
                                  (1+ (point)))
                                (current-buffer)))

    (overlay-put overlay 'priority 0)

    (overlay-put overlay 'face 'bookmark-face)

    (overlay-put overlay 'buffer (current-buffer))))

;;mods to bookmark-bmenu-mode
(define-key bookmark-bmenu-mode-map [return] 'bookmark-bmenu-select)
(define-key bookmark-bmenu-mode-map "q" 'delete-window)
(define-key bookmark-bmenu-mode-map [?\C-b] 'delete-window)


(provide 'bookmark+)




