;;; bookmark-manager.el --- Tool to help browse Netscape/Mozilla bookmarks.

;; $Id: bookmark-manager.el,v 1.7 2003/01/10 05:57:59 burton Exp $

;; Copyright (C) 1997-2000 Free Software Foundation, Inc.

;; Author: Kevin A. Burton (burton@openprivacy.org)
;; Maintainer: Kevin A. Burton (burton@openprivacy.org)
;; Location: http://relativity.yi.org
;; Keywords: backup
;; Version: 1.0

;; This file is [not yet] part of GNU Emacs.

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to the
;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; Needed a way to help manage bookmarks.  I also wanted to add some lisp magic
;; so that I can view bookmarks that are text (text/plain) *directly* in Emacs
;; and not in Mozilla but have everything else rendered in Mozilla.

;; I have not tested this on Netscape Navigator 4.x or earlier.  I do not use
;; Netscape Navigator.  I have looked at the format and it is the same.
;; Feedback on how bookmark-manager works here would be a Good Thing.

;;; ******** Format notes ********

;; Mozilla uses the following format:

;; <DT> -> mark new folder/url begin (not terminated)
   ;; if it contains an <H3>this is a folder
   ;; if it contains an <A HREF="">this is an URL

;; <DL> -> contents of folder (terminated by </DL>

;;; ******** Parsing notes ********

;; The working variable bm-level keeps track of the current
;; level.  First we set to the folder the user has requested to browse/complete
;; Then the level is set to 0.  URLs and folders are only entered if the level
;; is 0.

;;;

;;; ******** TODO ********

;; - change this package name to mbm.el (Mozilla Bookmark Manager). 

;; add support for editing a remote URLs from the bookmarks.  This can come in
;; handy for text files or rfcs.  IE.. mbm-edit and mbm-browse.  Maybe a regexp
;; matching scheme so that I can 



;; - Escape certain characters so that they become ASCII instead of escaped:
;;   -   &gt;     >
;;   -   &lt;     <
;;   -   &amp;    &
;;   -   &quot;   "

;; - If a file:// URL is present it should be possible to do extended things:
;;   - if it is a directory... run dired.


;; - make sure the *Bookmark Manager* is read-only.

;; - add support for searching your bookmark file.


;;; History:
;; 

(require 'browse-url)

;;FIXME: what should we do if both files are present???
;;; Code:
(defvar bm-files
  [ "~/.netscape/bookmarks.html" "~/.mozilla/default/bookmarks.html" ]
  "Ordered location(s) to search for Netscape/Mozilla bookmarks.")

(defvar bm-real-file
  nil
  "The file to use for your bookmark store.  If non-nil then `bm-files' is not used.")

(defvar bm-buffer-name "*Bookmark Manager*" "Buffer to use for reading your bookmarks.")

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; BEGIN TOKEN CONSTANTS
;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defvar bm-string-folder-begin  "<DT><H3"
  "String which marks the begining of a Folder.")

(defvar bm-string-folder-end  "</DL><p>"
  "String which marks the end of the folder")

(defvar bm-string-url-begin
  "<DT><A HREF=\""
  "String which marks the begining of a Folder.")

(defvar bm-string-folder-contents-begin
  "<DL><p>"
  "Marks the contents of a folder.")

(defvar bm-data '()
  "Bookmark entries to URLs.  Stores a recursive array of folders.  Each folder
is a list of  2 elements lists.  The root folder has a nil folder name.

(\"folder\" (\"name1\" \"http://url1\" ) (\"name2\" \"http://url2\" ))")



;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; END TOKEN CONSTANTS
;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;




;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; BEGIN REGEXP CONSTANTS
;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defvar bm-regexp-http-url "\\(http://[0-9a-zA-Z:\\.]+[0-9a-zA-Z\\.\\+/?,=%~_$&:#-]+\\)")
                                        
(defvar bm-regexp-file-url "\\(file:/+[0-9a-zA-Z\\./~#()_-]+\\)")

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; END REGEXP CONSTANTS
;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defun bm-browse-url(&optional folder)
  "Prompt for a bookmark and launch its URL via `browse-url'.  The optional
parameter folder is used to select the directory to browse."
  (interactive)

  ;;(bm-init)

  (bm-completing-read folder)
  
  )


(defun bm-completing-read(folder)

  (let(i root folder-name url-name (bookmark-list '()))

    ;;determine waht root folder to use.

    (setq root bm-data)

    ;;create a completion list.
    (setq i 1)
    (while (< i (length root))

      ;;if the current object is a folder... add it
      (let(obj)

        (setq obj (elt root i))

        (if (bm-folder-p obj)
          (progn

            (setq folder-name (concat (elt obj 0)
                                      "/"))
            
            (add-to-list 'bookmark-list (list folder-name i)))))
      
      (setq i (1+ i)))

    (setq my-test-list bookmark-list)
    (completing-read "Bookmark: " bookmark-list nil t)))
  
(defun bm-folder-p(object)
  "Return true if the object given is a folder."
  
  (not (bm-url-p object)))
   

(defun bm-url-p(object)
  "Return true if the object given is a url."

  ;;true if it is a list and each param is a string.
  (and (listp object)
       (= 2 (length object))
       (stringp (elt object 0))
       (stringp (elt object 1))))
  
(defun bm-scan()

  (set-buffer bm-buffer-name)
  (goto-char (point-min))

  (let((current-folder-path '()))

    ;;search for folders or urls
    (while (re-search-forward (concat bm-string-folder-begin
                                      "\\|"
                                      bm-string-folder-end
                                      "\\|"
                                      bm-string-url-begin) nil t)

      ;;process begin folders
      (if (string-equal (match-string 0)
                        bm-string-folder-end)
          (progn 

            ;;delete the last folder from the current-folder-path.
            (delete (length current-folder-path) current-folder-path)

            (message "FIXME: end of folder")))
      
      ;;process begin folders
      (if (string-equal (match-string 0)
                        bm-string-folder-begin)
          (progn 

            ;;go to the end of the match...
            (goto-char (match-end 0))

            ;;now get the actual folder data
            (if (re-search-forward ">" nil t)
                (let(folder)

                  (goto-char (match-end 0))

                  (re-search-forward "[^<]*" nil t)

                  ;;folder name should be the match string...
                  (setq folder (match-string 0))

                  ;;add this folder to the list...
                  (add-to-list 'current-folder-path folder)))))

      ;;process urls.
      (if (string-equal (match-string 0)
                        bm-string-url-begin)
          (progn 
            (goto-char (1+ (match-end 0)))

            (if (re-search-forward "[^\"]*" nil t)
                
                (let(url name)
                  (setq url (match-string 0))
                  ;;(message url)

                  ;;now pull out the name of this bookmark url...
                  (if (re-search-forward ">" nil t)
                      (progn
                        (goto-char (match-end 0))
                        (re-search-forward "[^<]*" nil t)

                        ;;we now have the name and the url...
                        (setq name (match-string 0))

                        ))
                                   
                  

                  )))
        
        


        ))))
  

(defun bm-path(path)
  "Return the given path list as a string which represents the path."

  )

(defun bm-init()
  "Setup the buffer to read from and open bookmarks.html"

  (let (file)
    
    (setq file (bm-get-file))

    (if (not (null file))
        (bm-init-buffer file)))

  ;; always to go the beginning of the temp buffer
  (bm-scan))
  

(defun bm-init-buffer(file)
  "Open the bookmark file and save it in the special buffer"

  (set-buffer (get-buffer-create bm-buffer))
  (erase-buffer) ;;just in case.
  
  (insert-file file))

(defun bm-get-file()
  "Get the correct bookmark file to use and return it"

  ;;if bm-real-file is null then compute what it should be.
  (if (null bm-real-file)
      (let (i current-file)
        (setq i 0)
        (while (< i (length bm-files))

          (setq current-file (aref bm-files i))

          (if (file-readable-p current-file)
              (setq bm-real-file current-file))

          (setq i (+ i 1)))))
  
  ;; if it is still nill then the file could not be found :(

  (if (null bm-real-file)
      (message "Could not find a bookmark on your system"))

    (message bm-real-file))

(provide 'bookmark-manager)

;;; bookmark-manager.el ends here
