;;; efindfile.el --- Smarter file opening using Tramp and find-file

;; Copyright (C) 1997-2000 Free Software Foundation, Inc.

;; Author: Kevin A. Burton (burton@openprivacy.org)
;; Maintainer: Kevin A. Burton (burton@openprivacy.org)
;; Location: http://relativity.yi.org
;; Keywords: Tramp, find-file, root, su
;; Version: 1.0

;; This file is [not yet] part of GNU Emacs.

;; This program is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free Software
;; Foundation; either version 2 of the License, or any later version.
;;
;; This program is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License along with
;; this program; if not, write to the Free Software Foundation, Inc., 59 Temple
;; Place - Suite 330, Boston, MA 02111-1307, USA.

;;; Commentary:

;; Some lisp magic to add an enhanced find-file to emacs.  Basically if the file
;; you are about to open doesn't have security setup correctly, either because
;; you are not root or it is under another user account, you will be prompted
;; with a user/password and then the operation will proceed normally.  This
;; should allow seemless operation within emacs.

;; There are some downsides to using this method for file access.  Since tramp
;; is magically triggered, file-browsing might not work if you get an
;; 'access-denied' message while browsing the filesystem



;;; TODO:
;;
;; - test both READ and WRITE violations to trigger tramp.
;;
;;
;; - Tramp does NOT work with 'su' via the 'sum' method.  It gets really
;; confused with its lock/temp files..

(require 'tramp)

;;NOTE:  I am using sudo here.  Not all *NIX machines have sudo installed by default
;;; Code:
(defvar efindfile-tramp-method "sum" "Method to use to transfer files with Tramp.")

(defvar efindfile-default-user "root" "Default user.")
(defvar efindfile-default-host "localhost" "Default host.")

(defun find-file-check-access(&optional filename)
  "Same as the 'find-file' function.  Browse the filesystem and if you don't
   have access to this file use Tramp to prompt for user information"
  (interactive)

  ;;file-writable-p

  (setq filename (read-file-name "Find file: "))

  
  ;; if the user can't read or write the file... use tramp...
  (if (or (not (file-writable-p filename))
          (not (file-readable-p filename)))
      ;;ok.  the current user can't edit this.  use Tramp.
      (find-file-using-tramp filename)
      ;;else no need to use Tramp
    ;;(find-file filename)
    )

  ;;at this point you can open a file with find-file and not tramp.

  )

(defun find-file-using-tramp(filename &optional user)
  "Given a filename, prompt for the correct user and then use tramp to open the file"

  (beep)
  ;;determine what username to use
  (setq user (read-string (concat "'" (user-login-name) "' doesn't have access.  Specify new user: ")
                          efindfile-default-user))

  ;;now use tramp to open the file.
  (find-file (concat "/r@" efindfile-tramp-method ":" user "@" efindfile-default-host ":" filename)))

;;setup the correct key-binding to replace the default one bound to find-file

(provide 'efindfile)

;;; efindfile.el ends here
