;;; file-within-header.el --- file information within the Emacs 21 header.

;; $Id: file-within-header.el,v 1.3 2002/01/18 11:14:57 burton Exp $

;; Copyright (C) 2000-2003 Free Software Foundation, Inc.
;; Copyright (C) 2000-2003 Kevin A. Burton (burton@openprivacy.org)

;; Author: Kevin A. Burton (burton@openprivacy.org)
;; Maintainer: Kevin A. Burton (burton@openprivacy.org)
;; Location: http://relativity.yi.org
;; Keywords:
;; Version: 1.0.1

;; This file is [not yet] part of GNU Emacs.

;; This program is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free Software
;; Foundation; either version 2 of the License, or any later version.
;;
;; This program is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License along with
;; this program; if not, write to the Free Software Foundation, Inc., 59 Temple
;; Place - Suite 330, Boston, MA 02111-1307, USA.

;;; Commentary:
;;
;; Display file information (filename, mode, owner) within the Emacs '21
;; header-line.

;;; History:

;; Tue Oct 30 2001 01:32 PM (burton@openprivacy.org): "File: " has one space
;; inserted for greater clarity.

;; TODO
;;
;; - change this into a minor mode. file-header-mode

;;; Code:
(defun file-within-header()
  "Put the current file name on the header."
  (interactive)

  (let(file-string modes-string owner-string attributes filename)

    (setq file-string " File: ")

    (file-within-header-bold file-string)

    (setq modes-string "Modes: ")

    (file-within-header-bold modes-string)

    (setq owner-string "Owner: ")

    (file-within-header-bold owner-string)

    (if (buffer-file-name)
        (setq filename (buffer-file-name))
      (if default-directory
          (setq filename default-directory)
        (if (and (equal major-mode 'dired-mode)
                 dired-directory)
            (setq filename dired-directory))))
    
    (setq attributes (file-attributes filename))
    
    (if (and (boundp 'header-line-format)
             (null header-line-format))
        (setq header-line-format (concat file-string filename " "
                                         modes-string (file-within-header-get-modes attributes) " "
                                         owner-string (file-within-header-get-owner attributes))))))

(defun file-within-header-bold(string)
  "Make the given string bold."

  (put-text-property 0 (length string) 'face 'bold string))

(defun file-within-header-get-modes(attributes)
  
  (elt attributes 8))

(defun file-within-header-get-owner(attributes)

  (user-login-name (elt attributes 2)))

(add-to-list 'find-file-hooks 'file-within-header)

(provide 'file-within-header)

;;; file-within-header.el ends here
