;;; find-file-url.el --- find file:// URLs.

;; Copyright (C) 1997-2000 Free Software Foundation, Inc.

;; Author: Kevin A. Burton (burton@openprivacy.org)
;; Maintainer: Kevin A. Burton (burton@openprivacy.org)
;; Location: http://relativity.yi.org
;; Keywords: find file URLs in Emacs
;; Version: 1.0.0

;; This file is [not yet] part of GNU Emacs.

;; This program is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free Software
;; Foundation; either version 2 of the License, or any later version.
;;
;; This program is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License along with
;; this program; if not, write to the Free Software Foundation, Inc., 59 Temple
;; Place - Suite 330, Boston, MA 02111-1307, USA.

;;; Commentary:
;;
;; Similar to find-file but works with file:// URLs.

;;; Install:
;;
;; (require 'find-file-url) in your .emacs


;;; History:
;; 

;;; Code:
(defvar find-file-url-regexp "file:/+[0-9a-zA-Z\\./~#()_:-]+" "Regexp to use for a file URL.")

(defun find-file-url-at-point()
  "Find the file:// URL at the current point."
  (interactive)

  (save-excursion

    (let(end)
      (goto-char (point-at-eol))
      (setq end (point))

      (goto-char (point-at-bol))
      (if (re-search-forward find-file-url-regexp end t)
          (let(url)
            (setq url (match-string 0))

            (find-file-url url))
        (error "No file URL at point")))))

(defun find-file-url(&optional url)
  "Given a file:// URL, open it using find-file."
  (interactive)
  
  (if (null url)
      (setq url (read-string "File URL: ")))
  
  ;;find the URL passed.
  (let(file)
    (setq file (substring url (length "file://")))
    (find-file file)))

(provide 'find-file-url)

;;; find-file-url.el ends here
