;;; vertline.el --- Display a vertical line within a buffer.

;; Copyright (C) 1997-2000 Free Software Foundation, Inc.

;; Author: Kevin A. Burton (burton@openprivacy.org)
;; Maintainer: Kevin A. Burton (burton@openprivacy.org)
;; Location: http://relativity.yi.org
;; Keywords: vertical line
;; Version: 1.0

;; This file is [not yet] part of GNU Emacs.

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to the
;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; NOTES:

;; it should be possible to do this as follows:
;;
;; if the line is < 80 chars
;;    - create an extent at EOL
;;
;;    - set the 'after-string property to a string with the 'face property set at the correct point.


;; - Note that it might also be possible to use the 'after-change-functions or
;; 'before-change-functions to keep the line up to date.


;;; TODO:
;;
;; - Use fill-column as the default for vertline-column
;;
;; - ability to have multiple lines... (this could be done with a list of
;; integers??)


(defvar vertline-column fill-column "Column number to create the vertical line.")

(defface vertline-face '((t (:background "gray20")))
  "Face used to highlight the vertical line in a buffer.")

(defun vertline-create-initial-line()
  (interactive) ;;FIXME: shouldn't be interactive when we are done

  (save-excursion

    (goto-char (point-min))

    (let(num-lines current-line)

      (setq num-lines (count-lines (point-min) (point-max)))

      (setq current-line 1)
      
      (while (<= current-line num-lines)

        (let(overlay start end after-string)

          (goto-line current-line)
          
          (goto-char (point-at-eol))
          
          (setq start (point))

          (setq end start)
          
          (setq overlay (make-overlay start end))

          ;;(set-extent-property overlay 'face 'vertline-face)
          
          (set-extent-property overlay 'window (selected-window))

          (set-extent-property overlay 'after-string (vertline-get-after-string)))
        
        (setq current-line (1+ current-line))))))

(defun vertline-get-after-string()
  "Get a string that we can use after this line for the overlay."

  (save-excursion
    
    (let(begin end line-length after-string offset)

      (message "FIXME (debug): get-after-tring is at: %s" (point))
      
      (goto-char (point-at-bol))

      (setq begin (point))

      (message "FIXME (debug): begin: %s" begin)
      
      (goto-char (point-at-eol))

      (setq end (point))

      (message "FIXME (debug): end: %s" end)
      
      (setq line-length (- end begin))

      (if (< line-length vertline-column)
          (progn

            (setq offset (- vertline-column line-length))

            (message "FIXME (debug): line-length: %s" line-length)

            (message "FIXME (debug): offset: %s" offset)

            (setq after-string (make-string offset ? ))

            ;;set text properties for the face

            (let(prop-start prop-end)

              (setq prop-start (1- offset))

              (setq prop-end (1+ prop-start))
            
              (add-text-properties prop-start prop-end '(face vertline-face) after-string))
            
            (message "FIXME (debug): length is %s" (length after-string))
            
            ;;(setq after-string (concat after-string "|"))                                       
            
            after-string)
        nil))))
                                  
(provide 'vertline)
