;; Copyright (C) 1997-2000 Free Software Foundation, Inc.

;; Author: Kevin A. Burton (burton@openprivacy.org)
;; Maintainer: Kevin A. Burton (burton@openprivacy.org)
;; Location: http://relativity.yi.org
;; Keywords: wget
;; Version: 1.1.0

;; This file is [not yet] part of GNU Emacs.

;; This program is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free Software
;; Foundation; either version 2 of the License, or any later version.
;;
;; This program is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License along with
;; this program; if not, write to the Free Software Foundation, Inc., 59 Temple
;; Place - Suite 330, Boston, MA 02111-1307, USA.

;;; History:
;;
;; - Thu Sep 25 2003 07:01 PM (burton@peerfear.org): We also now save to a
;; tmpfile so that emacs doesn't have to insert the contents in real time and
;; burn CPU.  
;;
;; - Thu Sep 25 2003 06:42 PM (burton@peerfear.org): Migrated BACK to using
;; standard wget.  While this requires the use of an external command this is
;; still faster than wget.  Emacs sync IO sucks.
;; 
;; - Fri Nov 02 2001 01:43 AM (burton@openprivacy.org): it now uses 100%
;; elisp.. IE no external wget binary needed.
;;
;; - Tue Oct 30 2001 08:22 PM (burton@openprivacy.org): enable html-mode if we
;; are viewing an html file.
;;
;; - Mon Oct 29 2001 07:04 PM (burton@openprivacy.org): modernized for Emacs 21.

;; Version 1.0.1:
;;
;; - Correctly paying attention to stderr.  Invalid URLs will not be detected
;;   and the user notified.
;;
;; - Create a new *wget* buffer for each new URL and dont' reuse the same
;;   buffer.
;;
;; - supports --save-headers from wget
;;
;; - http:// is now specified as the init for input
;;
;; - have wget-mode that highlights the firsts
;;   lines of  the buffer until the regexp ^$

;; Version 1.0:
;; init.

;;; Commentary:

;; This is an interface for wget.  Basically it allows you to pull down URLs and
;; then view the output in a buffer.  This is a too to help develop/debug web
;; applications.

;;; TODO:

;; - This is NOT synchronous AKA if we have a server that is remove and times
;; out Emacs will allow the user to type things and the metaphor becomes VERY
;; confusing!  This needs to be fixed!
;; 

;; - FIXME: why are we getting ^M chars in the output?
;;
;;    - I think I need to set my character conversion

;; - include a mechanism to support posting content to a webserver.  This would
;; prompt for the URL, method (get|post), and any specific HTTP headers to send.
;; It should save this URL and all its params in memory for future use. The the
;; user can repeat this again.

(defface wget-headers-face '((t (:foreground "GoldenRod" :bold t)))
  "Face used to highlight the HTTP headers in the buffer.")

(defun wget(url)
  (interactive
   (list
    (read-string "URL: " "")))

;;   (shell-command (concat "wget --save-headers --output-document=/dev/stdout --non-verbose " 
;;                          url) 
;;                  (wget-get-buffer url) 
;;                  (wget-get-buffer url))

  (let((tmpfile (make-temp-file "wget.el")))
    
    (start-process "wget"
                   (wget-get-buffer url)
                   "wget"
                   "--save-headers"
                   (concat "--output-document=" tmpfile)
                   "--non-verbose"
                   url)
                 
    (find-file tmpfile)

    (message (concat "Fetching " url " ..."))))

(defun wget-init(url)
  "Clear the temp wget buffer if there is data in it"

  ;;switch to the correct buffer
  (view-buffer (wget-get-buffer url))

  ;;(mark-whole-buffer)

  ;;if the wget buffer is full reset it
  (if (> (buffer-size) 0)
      (delete-region (region-beginning) (region-end))))

(defun wget-get-buffer(url)
  ;;based on the url... get a correct temporary buffer.
  (concat "*wget " url "*"))

(defun wget-highlight-buffer()
  
  (save-excursion
    (insert "--------------------------------------------------------------------------------\n")
    (search-forward-regexp "^$" nil t)
    (insert "--------------------------------------------------------------------------------\n")
    (end-of-line)

    (setq current-overlay (make-overlay 1 (point)))
      
    (overlay-put current-overlay 'face 'wget-headers-face)
    
    (overlay-put current-overlay 'window (selected-window))))

(provide 'wget)

;;; wget.el ends here
