;;; xmms-shell.el --- quick hack to talk to xmms-shell 

; inspired by dired-xmms-play-mp3 from gnu.emacs.sources

;; copyright (C) 2002 Shae Matijs Erisson 
;; released under the Lesser GPL license.

;; Author:   Shae Erisson <shae@ScannedInAvian.com>
;; Keywords: multimedia xmms fnord mp3 eris ogg flac
;; Version:  0.2
;; X-CVS:    $Id: xmms-shell.el,v 1.4 2002/12/04 08:18:21 shae Exp $	

;;; Commentary:

; this is the second or third chunk of elisp I've written
; constructive criticism welcomed
; requires the xmms-shell, and therefore, xmms
; sadly, I discovered that xmms requires X (duh)
; so at some point I'll switch to something that works in the console

;;; Todo:
; set up all the commands to return status info from "xmms-shell -e status"
; volume changes should return current volume and max volume
; track changes should return title of the current track
; playlist-show should return a buffer that lets you hit enter on a track and
; start playing it 
; playlist-show should also let you see the extra info, path + etc
; playlist-sort ?
; what about editing playlists?
; add more commands
; delYsid suggested using xmms-shell as an interactive process?
; hack playlist-next and playlist-previous to be one function that takes -1 ?
; add C-s style song jumping, or iswitch style song jumping, can I use i-do
; code? 
; check for xmms-shell version and use old static shuffle/repeat for older
; versions? 
;
;xmms-shell> help
;General:
;  ?                - list commands and show their usage
;  echo             - display a string
;  exit             - terminate the shell
;  fade             - adjust volume to specified level over time
;  help             - list commands and show their usage
;  quit             - terminate the shell
;  status           - display the current status of XMMS
;  version          - display XMMS-Shell and XMMS version information
;  xmmsexit         - force XMMS and XMMS-Shell to exit
;  xmmsquit         - force XMMS and XMMS-Shell to exit

;Playback:
;  fakepause        - pause XMMS and release the output device
;  pause            - pause or unpause playback
;  play             - start or resume playback
;  repeat           - set repeat mode
;  shuffle          - set shuffle mode
;  stop             - stop playback

;Playlist:
;  backward         - advance backward in the playlist
;  clear            - clear the current playlist
;  current-track    - display current track
;  currenttrack     - display current track
;  forward          - advance forward in the playlist
;  jump             - jump to a position in the playlist
;  list             - display the playlist
;  load             - add music or playlist files to the playlist
;  next             - advance forward in the playlist
;  previous         - advance backward in the playlist
;  random-track     - jump to random track
;  randomtrack      - jump to random track
;  remove           - remove track(s) from the playlist
;  resetdevice      - force XMMS to reset the output device
;  save             - save playlist to file

;Volume Control:
;  +                - increase the volume in either or both channels
;  -                - decrease the volume in either or both channels
;  balance          - set or query balance level
;  band             - view or set the value of equalizer bands
;  downvolume       - decrease the volume in either or both channels
;  preamp           - view or set the equalizer's preamp value
;  upvolume         - increase the volume in either or both channels
;  volume           - set the volume in either or both channels to a
;                     given level

;Window Control:
;  eject            - cause XMMS to display its file load dialog
;  preferences      - cause XMMS to display its preferences dialog
;  window           - show or hide XMMS windows

;;; Code:


; just a bit of modularity
(defun xmms-shell-command (arg) 
  "(xmms-shell-command ARG)
sends the command in ARG to an xmms process via shell-command-to-string
currently no support for talking to one of multiple xmms processes
(the -s parameter for xmms-shell)"
  ()
  (shell-command-to-string (concat "xmms-shell -e " arg)))

; two days I spent trying to figure this out
; kensanata came up with it instantly
(defun xsc (cmd) `(lambda nil (interactive) (xmms-shell-command ,cmd)))

(defun xmms-shell-playlist-show ()
  "(xmms-shell-playlist-show)
creates or gets a buffer named \"*XMMS Playlist*\"
clears it, throws the playlist into it, then shows that buffer
there's probably a more efficient way to do this, dunno what it is"
  (interactive)
  (let
      ((xmms-playlist-buffer (get-buffer-create "*XMMS Playlist*")))
    (progn
      (erase-buffer xmms-playlist-buffer)
      (insert-string (xmms-shell-command "list") xmms-playlist-buffer)
      (pop-to-buffer xmms-playlist-buffer))))

(defun xmms-shell-playlist-jump (num) (interactive "snum? ")
  "(xmms-shell-playlist-jump NUM)
jump         - jump to a position in the playlist
as a shell command it has to look like \"xmms -e \"jump 91\"\""
  (xmms-shell-command (concat "\"jump " num "\"")))

;shuffle stuff
; xmms-shell-playlist-shuffle-state gets set to a known state on startup
; because I can't ask "status" about shuffle state
(defun xmms-shell-playlist-shuffle (arg) (interactive)
					;shuffle      - set shuffle mode
  (xmms-shell-command (concat "\"shuffle " arg "\"")))

(fset 'xmms-shell-playlist-shuffle-on (xsc "shuffle on"))
(fset 'xmms-shell-playlist-shuffle-off (xsc "shuffle off"))

(defun xmms-shell-playlist-shuffle-toggle () (interactive)
  (if (string-equal (cadr (cdddr (xmms-shell-status-get))) "on")
      (xmms-shell-playlist-shuffle-off)
    (xmms-shell-playlist-shuffle-on)))

; repeat stuff
(fset 'xmms-shell-playlist-repeat-on (xsc "repeat on"))
(fset 'xmms-shell-playlist-repeat-off (xsc "repeat off"))

(defun xmms-shell-playlist-repeat-toggle () (interactive)
  (if (string-equal (cadddr (xmms-shell-status-get)) "on")
      (xmms-shell-playlist-repeat-off)
    (xmms-shell-playlist-repeat-on)))



(defun xmms-shell-status-get () (interactive)
; ugLEE - there must be a nicer way to do this
; if you find it, tell me!
  (let 
      ((xmms-shell-status-string (xmms-shell-command "status"))
       (xmms-shell-status-regexp "Playing: \\(.*\\) \\(?:\(.*\)\\)?\nTime: \\(.*\\)\nRepeat mode: \\(.*\\)\nShuffle mode: \\(.*\\)\nBalance: \\(.*\\)\nSkin: \\(.*\\)\nLeft volume: \\(.*\\)\nRight volume: \\(.*\\)\n.*\n.*\n.*"))
    (progn
      (string-match xmms-shell-status-regexp  xmms-shell-status-string)
      (let
	  ((track-name (match-string 1 xmms-shell-status-string))
	   (track-time (match-string 2 xmms-shell-status-string))
           (repeat-mode (match-string 3 xmms-shell-status-string))
           (shuffle-mode (match-string 4 xmms-shell-status-string))
	   (volume-left (match-string 7 xmms-shell-status-string))
	   (volume-right (match-string 8 xmms-shell-status-string)))
					; and here's what we return, (track-name track-time (volume))
					; volume is only one number if both channels have equal volume, two if they're unequal
					; best way I could think of to deal with this
	(list track-name track-time (if (string-equal volume-right volume-left)
					(list volume-left)
				      (list volume-left volume-right))
              repeat-mode shuffle-mode)
	))))

(defun xmms-shell-status-show () (interactive)
  (let* ((status-list (xmms-shell-status-get))
	 (track-name (car status-list))
	 (track-time (cadr status-list))
	 (volume-pair (caddr status-list)))
    (message (concat track-name " " track-time " Volume: " 
; I'm not happy with the way Volume works, it's better, but still ugly
		     (if (eq 1 (length volume-pair)) (car volume-pair) (concat (car volume-pair) "/" (cadr volume-pair)))
		     " of 100" " Repeat: " (cadddr status-list) " Shuffle: " (cadr (cdddr status-list))))))

;hyperactivity?
; I like the Hyper key
; here's what I'm using at the moment
; note that I actually call this defun in my .emacs
; so if you put it in your own keybindings, you'll need to execute it yourself
(defun shae-xmms-keymap () (interactive)
  (progn
    (global-set-key [(hyper down)] (xsc "-"))
    (global-set-key [(hyper up)] (xsc "+")) ; increase volume 1%
    (global-set-key [(hyper x) l] 'xmms-shell-playlist-show)
    (global-set-key [(hyper right)] (xsc "forward"))
    (global-set-key [(hyper left)] (xsc "backward"))
    (global-set-key [(hyper b)] (xsc "pause"))
    (global-set-key [(hyper x) j] 'xmms-shell-playlist-jump)
    (global-set-key [(hyper x) i] 'xmms-shell-status-show) ; i = info?
    (global-set-key [(hyper x) p] (xsc "play"))
    (global-set-key [(hyper x) s] (xsc "stop"))
    (global-set-key [(hyper x) t s] 'xmms-shell-playlist-shuffle-toggle)
    (global-set-key [(hyper x) s] (xsc "save"))
    ))

(provide 'xmms-shell)

;; xmms-shell ends here

